@@ -1,20 +1,39 @@
 /*
- * tf_util.c
+ * lib/krb4/tf_util.c
  *
- * Copyright 1987, 1988 by the Massachusetts Institute of Technology.
+ * Copyright 1985, 1986, 1987, 1988, 2000, 2001 by the Massachusetts
+ * Institute of Technology.  All Rights Reserved.
  *
- * For copying and distribution information, please see the file
- * <mit-copyright.h>.
+ * Export of this software from the United States of America may
+ *   require a specific license from the United States Government.
+ *   It is the responsibility of any person or organization contemplating
+ *   export to obtain such a license before exporting.
+ * 
+ * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
+ * distribute this software and its documentation for any purpose and
+ * without fee is hereby granted, provided that the above copyright
+ * notice appear in all copies and that both that copyright notice and
+ * this permission notice appear in supporting documentation, and that
+ * the name of M.I.T. not be used in advertising or publicity pertaining
+ * to distribution of the software without specific, written prior
+ * permission.  Furthermore if you modify this software you must label
+ * your software as modified software and not distribute it in such a
+ * fashion that it might be confused with the original M.I.T. software.
+ * M.I.T. makes no representations about the suitability of
+ * this software for any purpose.  It is provided "as is" without express
+ * or implied warranty.
  */
 
-#include "mit-copyright.h"
-
 #include "krb.h"
 #include "k5-int.h"
+#include "krb4int.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <sys/stat.h>
 #include <fcntl.h>
 
@@ -28,7 +47,6 @@
 #define TF_LCK_RETRY ((unsigned)2)	/* seconds to sleep before
 					 * retry if ticket file is
 					 * locked */
-extern int errno;
 extern int krb_debug;
 
 void tf_close();
@@ -44,7 +62,6 @@ char *shmat();
 #ifdef NEED_UTIMES
 
 #include <sys/time.h>
-#include <unistd.h>
 #ifdef __SCO__
 #include <utime.h>
 #endif
@@ -62,6 +79,20 @@ int utimes(path, times)
 }
 #endif
 
+#ifdef HAVE_SETEUID
+#define do_seteuid(e) seteuid((e))
+#else
+#ifdef HAVE_SETRESUID
+#define do_seteuid(e) setresuid(-1, (e), -1)
+#else
+#ifdef HAVE_SETREUID
+#define do_seteuid(e) setreuid(geteuid(), (e))
+#else
+#define do_seteuid(e) (errno = EPERM, -1)
+#endif
+#endif
+#endif
+
 /*
  * fd must be initialized to something that won't ever occur as a real
  * file descriptor. Since open(2) returns only non-negative numbers as
@@ -80,7 +111,7 @@ static int  curpos;			/* Position in tfbfr */
 static int  lastpos;			/* End of tfbfr */
 static char tfbfr[BUFSIZ];		/* Buffer for ticket data */
 
-static tf_gets(), tf_read();
+static int tf_gets (char *, int), tf_read (char *, int);
 
 /*
  * This file contains routines for manipulating the ticket cache file.
@@ -98,15 +129,22 @@ static tf_gets(), tf_read();
  *      Where "CREDENTIAL_x" consists of the following fixed-length
  *      fields from the CREDENTIALS structure (see "krb.h"):
  *
- *              char            service[ANAME_SZ]
- *              char            instance[INST_SZ]
- *              char            realm[REALM_SZ]
+ *              string          service[ANAME_SZ]
+ *              string          instance[INST_SZ]
+ *              string          realm[REALM_SZ]
  *              C_Block         session
  *              int             lifetime
  *              int             kvno
  *              KTEXT_ST        ticket_st
  *              long            issue_date
  *
+ * Strings are stored NUL-terminated, and read back until a NUL is
+ * found or the indicated number of bytes have been read.  (So if you
+ * try to store a string exactly that long or longer, reading them
+ * back will not work.)  The KTEXT_ST structure is stored as an int
+ * length followed by that many data bytes.  All ints are stored using
+ * host size and byte order for "int".
+ *
  * Short description of routines:
  *
  * tf_init() opens the ticket file and locks it.
@@ -144,12 +182,12 @@ static tf_gets(), tf_read();
  * TKT_FIL_LCK  - couldn't lock the file, even after a retry
  */
 
-int tf_init(tf_name, rw)
-    char   *tf_name;
+int KRB5_CALLCONV tf_init(tf_name, rw)
+    const char   *tf_name;
     int rw;
 {
     int     wflag;
-    uid_t   me= getuid();
+    uid_t   me, metoo;
     struct stat stat_buf, stat_buffd;
 #ifdef TKT_SHMEM
     char shmidname[MAXPATHLEN]; 
@@ -163,6 +201,7 @@ int tf_init(tf_name, rw)
     }
 
     me = getuid();
+    metoo = geteuid();
 
     switch (rw) {
     case R_TKT_FIL:
@@ -196,8 +235,30 @@ int tf_init(tf_name, rw)
     curpos = sizeof(tfbfr);
 
 #ifdef TKT_SHMEM
+    if (lstat(shmidname, &stat_buf) < 0) {
+	switch (errno) {
+	case ENOENT:
+	    return NO_TKT_FIL;
+	default:
+	    return TKT_FIL_ACC;
+	}
+    }
+    if (stat_buf.st_uid != me || !(stat_buf.st_mode & S_IFREG)
+	|| stat_buf.st_nlink != 1 || stat_buf.st_mode & 077) {
+	return TKT_FIL_ACC;
+    }
+
+    /*
+     * Yes, we do uid twiddling here.  It's not optimal, but some
+     * applications may expect that the ruid is what should really own
+     * the ticket file, e.g. setuid applications.
+     */
+    if (me != metoo && do_seteuid(me) < 0)
+	return KFAILURE;
     sfp = fopen(shmidname, "r");	/* only need read/write on the
 					   actual tickets */
+    if (me != metoo && do_seteuid(metoo) < 0)
+	return KFAILURE;
     if (sfp == 0) {
         switch(errno) {
         case ENOENT:
@@ -207,10 +268,11 @@ int tf_init(tf_name, rw)
 	}
     }
 
-    /* lstat() and fstat() the file to check that the file we opened is the *
-     * one we think it is, and to check ownership.                          */
-    if ((fstat(sfp->_file, &stat_buffd) < 0) || 
-	(lstat(shmidname, &stat_buf) < 0)) {
+    /*
+     * fstat() the file to check that the file we opened is the one we
+     * think it is.
+     */
+    if (fstat(fileno(sfp), &stat_buffd) < 0) {
         (void) close(fd);
 	fd = -1;
 	switch(errno) {
@@ -271,8 +333,25 @@ int tf_init(tf_name, rw)
     tmp_shm_addr = krb_shm_addr;
 #endif /* TKT_SHMEM */
     
+    if (lstat(tf_name, &stat_buf) < 0) {
+	switch (errno) {
+	case ENOENT:
+	    return NO_TKT_FIL;
+	default:
+	    return TKT_FIL_ACC;
+	}
+    }
+    if (stat_buf.st_uid != me || !(stat_buf.st_mode & S_IFREG)
+	|| stat_buf.st_nlink != 1 || stat_buf.st_mode & 077) {
+	return TKT_FIL_ACC;
+    }
+
     if (wflag) {
+	if (me != metoo && do_seteuid(me) < 0)
+	    return KFAILURE;
 	fd = open(tf_name, O_RDWR, 0600);
+	if (me != metoo && do_seteuid(metoo) < 0)
+	    return KFAILURE;
 	if (fd < 0) {
 	    switch(errno) {
 	    case ENOENT:
@@ -281,10 +360,11 @@ int tf_init(tf_name, rw)
 	        return TKT_FIL_ACC;
 	  }
 	}
-	/* lstat() and fstat() the file to check that the file we opened is the *
-	 * one we think it is, and to check ownership.                          */
-	if ((fstat(fd, &stat_buffd) < 0) || 
-	    (lstat(tf_name, &stat_buf) < 0)) {
+	/*
+	 * fstat() the file to check that the file we opened is the
+	 * one we think it is, and to check ownership.
+	 */
+	if (fstat(fd, &stat_buffd) < 0) {
 	    (void) close(fd);
 	    fd = -1;
 	    switch(errno) {
@@ -327,7 +407,11 @@ int tf_init(tf_name, rw)
      * for read-only operations and locked for shared access. 
      */
 
+    if (me != metoo && do_seteuid(me) < 0)
+	return KFAILURE;
     fd = open(tf_name, O_RDONLY, 0600);
+    if (me != metoo && do_seteuid(metoo) < 0)
+	return KFAILURE;
     if (fd < 0) {
         switch(errno) {
 	case ENOENT:
@@ -336,10 +420,11 @@ int tf_init(tf_name, rw)
 	    return TKT_FIL_ACC;
 	}
     }
-    /* lstat() and fstat() the file to check that the file we opened is the *
-     * one we think it is, and to check ownership.                          */
-    if ((fstat(fd, &stat_buffd) < 0) || 
-	(lstat(tf_name, &stat_buf) < 0)) {
+    /*
+     * fstat() the file to check that the file we opened is the one we
+     * think it is, and to check ownership.
+     */
+    if (fstat(fd, &stat_buffd) < 0) {
         (void) close(fd);
 	fd = -1;
 	switch(errno) {
@@ -387,7 +472,7 @@ int tf_init(tf_name, rw)
  * was longer than ANAME_SZ, TKT_FIL_FMT is returned. 
  */
 
-int tf_get_pname(p)
+int KRB5_CALLCONV tf_get_pname(p)
     char   *p;
 {
     if (fd < 0) {
@@ -410,7 +495,7 @@ int tf_get_pname(p)
  * instance may be null. 
  */
 
-int tf_get_pinst(inst)
+int KRB5_CALLCONV tf_get_pinst(inst)
     char   *inst;
 {
     if (fd < 0) {
@@ -434,7 +519,7 @@ int tf_get_pinst(inst)
  * EOF          - end of file encountered
  */
 
-int tf_get_cred(c)
+int KRB5_CALLCONV tf_get_cred(c)
     CREDENTIALS *c;
 {
     KTEXT   ticket = &c->ticket_st;	/* pointer to ticket */
@@ -501,7 +586,7 @@ int tf_get_cred(c)
  * The return value is not defined.
  */
 
-void tf_close()
+void KRB5_CALLCONV tf_close()
 {
     if (!(fd < 0)) {
 #ifdef TKT_SHMEM
@@ -603,8 +688,6 @@ tf_read(s, n)
     return n;
 }
      
-char   *tkt_string();
-
 /*
  * tf_save_cred() appends an incoming ticket to the end of the ticket
  * file.  You must call tf_init() before calling tf_save_cred().
@@ -632,7 +715,7 @@ int tf_save_cred(service, instance, realm, session, lifetime, kvno,
 {
 
     off_t   lseek();
-    int     count;		/* count for write */
+    unsigned int count;		/* count for write */
 #ifdef TKT_SHMEM
     int	    *skey_check;
 #endif /* TKT_SHMEM */
