@@ -12,4 +12,4 @@
 
 /* Format: "KRB5_BRAND: <cvs tag> <date>" */
 
-static char krb5_brand[] = "KRB5_BRAND: krb5-1-3-final 1.3 20030708";
+static char krb5_brand[] = "KRB5_BRAND: krb5-1-3-1-final 1.3.1 20030730";
